# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc
from . import rpc_data_pb2 as rpc__data__pb2


class GreeterStub(object):
  """The greeting service definition.
  """

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.handle_request = channel.unary_unary(
        '/rpc_data.Greeter/handle_request',
        request_serializer=rpc__data__pb2.Request.SerializeToString,
        response_deserializer=rpc__data__pb2.Reply.FromString,
        )


class GreeterServicer(object):
  """The greeting service definition.
  """

  def handle_request(self, request, context):
    """Sends a greeting
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_GreeterServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'handle_request': grpc.unary_unary_rpc_method_handler(
          servicer.handle_request,
          request_deserializer=rpc__data__pb2.Request.FromString,
          response_serializer=rpc__data__pb2.Reply.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'rpc_data.Greeter', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))
